#
# admin.py - Controller for the admin interface
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>

from flask import Blueprint, render_template, redirect, url_for, request
from flask.ext.wtf import Form, SelectField
from wtforms.ext.sqlalchemy.orm import model_form

from sqlalchemy import or_

from blockerbugs import app, db
from blockerbugs.controllers.forms import MilestoneForm, SpinForm
from blockerbugs.controllers.users import admin_required
from blockerbugs.models.milestone import Milestone
from blockerbugs.models.release import Release
from blockerbugs.models.update import Update
from blockerbugs.models.spin import Spin, SpinState

admin = Blueprint('admin', __name__)

# forms
class CurrentReleaseForm(Form):
    release = SelectField(u'Current Milestone', coerce=int)


ReleaseForm = model_form(Milestone, Form)


def get_current_release():
    current_release = Milestone.query.filter_by(current=True).first()
    if current_release:
        return current_release.name
    else:
        return ''


def set_current_release(new_release_id):
    new_release = Milestone.query.filter_by(id=new_release_id).first()
    current_releases = Milestone.query.filter_by(current=True).all()
    for current_release in current_releases:
        if current_release.id == new_release_id:
            continue
        current_release.current = False
        db.session.add(current_release)
    new_release.current = True
    db.session.add(new_release)
    db.session.commit()


@admin.route('/admin/main', methods=("GET", "POST"))
@admin_required
def admin_index():
    if app.debug:
        app.logger.debug('rendering index')
    milestones = Milestone.query.all()
    spins = Spin.query.order_by(Spin.date_requested.desc()).all()
    releases = Release.query.all()
    current_relform = CurrentReleaseForm()
    current_relform.release.choices = [(m.id, "Fedora %d %s" % (m.release.number, m.version)) for m in Milestone.query.filter_by(active=True).all()]
    if current_relform.validate_on_submit():
        app.logger.debug('got %s as the new current release' % current_relform.release.data)
        set_current_release(current_relform.release.data)
        return redirect(url_for(".admin_index"))
    else:
        app.logger.debug('relform did not validate for request type %s' % request.method)
    return render_template('admin/main.html', form=current_relform,
                            milestones=milestones, spins=spins, releases=releases)


@admin.route('/admin/release', methods=("GET", "POST"))
@admin_required
def release():
    milestones = Milestone.query.all()
    current_milestone = Milestone.query.filter_by(current=True).first()
    if current_milestone:
        current = current_milestone.name
    else:
        current = ''

    newform = ReleaseForm(request.form)
    if newform.validate_on_submit():
        new_release = Milestone(newform.number.data, newform.version.data,
                                newform.blocker_tracker.data,
                                newform.fe_tracker.data, newform.name.data,
                                active=newform.active.data, current=False)
        db.session.add(new_release)
        db.session.commit()
        return redirect(url_for(".admin_index"))
    return render_template('admin/add_release.html', milestones=milestones,
                           current=current, newform=newform)


@admin.route('/admin/milestone', methods=("GET", "POST"))
@admin_required
def milestone():
    milestones = Milestone.query.all()
    current_milestone = Milestone.query.filter_by(current=True).first()
    if current_milestone:
        current = current_milestone.name
    else:
        current = ''

    newform = MilestoneForm(request.form)
    newform.release.choices = Release.query.filter_by(active=True).all()
    if newform.validate_on_submit():
        new_release = Milestone(newform.number.data, newform.version.data,
                                newform.blocker_tracker.data,
                                newform.fe_tracker.data, newform.name.data,
                                active=newform.active.data, current=False)
        db.session.add(new_release)
        db.session.commit()
        return redirect(url_for(".admin_index"))
    return render_template('admin/add_release.html', milestones=milestones,
                           current=current, newform=newform)


@admin.route('/admin/milestone/<int:number>/<version>',
             methods=('GET', 'POST'))
@admin_required
def modify_milestone(number, version):
    release = Release.query.filter_by(number=number).first()
    milestone = Milestone.query.filter_by(release=release, version=version).first()
    rel_form = MilestoneForm(request.form, milestone)
    rel_form.release.choices = [(r.id, r.number) for r in Release.query.all()]
    rel_form.succeeds.choices = [(m.id, m.name) for m in Milestone.query.filter_by(active=True).all()]
    if rel_form.validate_on_submit():
        # this really isn't finished yet - doing it the wrong way
        # will fix after update table modifications are done
        selected_release = Release.query.filter_by(id=rel_form.release.data).first()
        if milestone.release != selected_release:
            milestone.release = selected_release
        if milestone.blocker_tracker != rel_form.blocker_tracker.data:
            milestone.blocker_tracker = rel_form.blocker_tracker.data
        if milestone.fe_tracker != rel_form.fe_tracker.data:
            milestone.fe_tracker = rel_form.fe_tracker.data
        if milestone.active != rel_form.active.data:
            milestone.active = rel_form.active.data
        db.session.add(milestone)
        db.session.commit()
        return redirect(url_for('.admin_index'))
    return render_template('admin/modify_release.html', form=rel_form, rel_num=release.number)


@admin.route('/admin/spin', methods=('GET', 'POST'))
@admin_required
def spin():
    spinform = SpinForm(request.form)
    spinform.milestone.choices = [(m.id, m.name) for m in Milestone.query.filter_by(active=True).all()]
    spinform.updates.choices = [(u.id, u.title) for u in Update.query.filter(or_(Update.status!='stable', Update.pending),
                                    Update.status != 'obsolete').all()]
    spinform.succeeds.choices = [(s.id, s.name) for s in Spin.query.all()]
    spinform.succeeds.choices.insert(0, (-1, ''))
    spinform.state.choices = [(key, description) for key, description in SpinState]

    if spinform.validate_on_submit():
        selected_updates = [Update.query.filter_by(id=i).first() for i in spinform.updates.data]
        selected_milestone = Milestone.query.filter_by(id=spinform.milestone.data).first()
        if spinform.succeeds.data == -1:
            succeeds_list = []
        else:
            succeeds_list = [Spin.query.filter_by(id=spinform.succeeds.data).first()]
        new_spin = Spin(spinform.name.data, spinform.spin_type.data, selected_updates,
                        selected_milestone, succeeds_list, url=spinform.url.data,
                        state=SpinState.from_string(spinform.state.data))
        db.session.add(new_spin)
        db.session.commit()
        return redirect(url_for(".admin_index"))
    else:
        if spinform.errors:
            for error in spinform.errors:
                print error

    return render_template('admin/add_spin.html', spinform=spinform,
                           postpoint=url_for('admin.spin'))


@admin.route('/admin/spin/<name>', methods=('GET', 'POST'))
@admin_required
def show_spin(name):
    spin = Spin.query.filter_by(name=name).first()
    spinform = SpinForm(request.form, spin)

    spinform.milestone.choices = [(m.id, m.name) for m in Milestone.query.filter_by(active=True).all()]
    spinform.updates.choices = [(u.id, u.title) for u in Update.query.filter(or_(Update.status!='stable', Update.pending),
                                    Update.status != 'obsolete').all()]
    spinform.succeeds.choices = [(s.id, s.name) for s in Spin.query.all()]
    spinform.succeeds.choices.insert(0, (-1, ''))
    spinform.state.choices = [(key, description) for key, description in SpinState]

    if spinform.validate_on_submit():
        if spinform.succeeds.data == -1:
            spin.succeeds = []
        else:
            spin.succeeds = [Spin.query.filter_by(id=spinform.succeeds.data).first()]
        spin.updates = [Update.query.filter_by(id=i).first() for i in spinform.updates.data]
        spin.milestone = Milestone.query.filter_by(id=spinform.milestone.data).first()
        spin.state = SpinState.from_string(spinform.state.data)
        spin.url = spinform.url.data
        db.session.add(spin)
        db.session.commit()

    spinform.updates.data = [u.id for u in spin.updates]
    if len(spin.succeeds) > 0:
        spinform.succeeds.data = spin.succeeds[0].id
    spinform.state.data = spin.state.value

    return render_template('admin/add_spin.html', spinform=spinform,
                           postpoint=url_for('admin.show_spin', name=name))
